@@ -259,10 +259,11 @@ CLUSTER_DEFAULT_VALUE = (
 ACTION_STATUSES = (
     ACTION_INIT, ACTION_WAITING, ACTION_READY, ACTION_RUNNING,
     ACTION_SUCCEEDED, ACTION_FAILED, ACTION_CANCELLED,
-    ACTION_WAITING_LIFECYCLE_COMPLETION,
+    ACTION_WAITING_LIFECYCLE_COMPLETION, ACTION_SUSPENDED,
 ) = (
     'INIT', 'WAITING', 'READY', 'RUNNING',
-    'SUCCEEDED', 'FAILED', 'CANCELLED', 'WAITING_LIFECYCLE_COMPLETION'
+    'SUCCEEDED', 'FAILED', 'CANCELLED', 'WAITING_LIFECYCLE_COMPLETION',
+    'SUSPENDED',
 )
 
 EVENT_LEVELS = {
@@ -274,10 +275,10 @@ EVENT_LEVELS = {
 }
 
 DETECTION_TYPES = (
-    LIFECYCLE_EVENTS, NODE_STATUS_POLLING,
+    LIFECYCLE_EVENTS, NODE_STATUS_POLLING, NODE_STATUS_POLL_URL,
     # LB_STATUS_POLLING,
 ) = (
-    'LIFECYCLE_EVENTS', 'NODE_STATUS_POLLING',
+    'LIFECYCLE_EVENTS', 'NODE_STATUS_POLLING', 'NODE_STATUS_POLL_URL',
     # 'LB_STATUS_POLLING',
 )
 
@@ -305,3 +306,9 @@ LIFECYCLE_TRANSITION_TYPE = (
 ) = (
     'termination',
 )
+
+VM_STATUS = (
+    VS_ACTIVE, VS_ERROR, VS_SUSPENDED, VS_SHUTOFF, VS_PAUSED, VS_RESCUE,
+) = (
+    'ACTIVE', 'ERROR', 'SUSPENDED', 'SHUTOFF', 'PAUSED', 'RESCUE',
+)
