@@ -19,7 +19,6 @@ from osprofiler import profiler
 
 from senlin.common import consts
 from senlin.common import exception
-from senlin.common.i18n import _
 from senlin.common import scaleutils
 from senlin.common import utils
 from senlin.engine.actions import base
@@ -67,7 +66,6 @@ class ClusterAction(base.Action):
         :returns: A tuple containing the result and the corresponding reason.
         """
         status = self.get_status()
-        reason = ''
         while status != self.READY:
             if status == self.FAILED:
                 reason = ('%(action)s [%(id)s] failed') % {
@@ -94,8 +92,8 @@ class ClusterAction(base.Action):
                     self.is_timeout(lifecycle_hook_timeout)):
                 # if lifecycle hook timeout is specified and Lifecycle hook
                 # timeout is reached, return
-                reason = _('%(action)s [%(id)s] lifecycle hook timeout') % {
-                    'action': self.action, 'id': self.id[:8]}
+                reason = ('%(action)s [%(id)s] lifecycle hook timeout'
+                          '') % {'action': self.action, 'id': self.id[:8]}
                 LOG.debug(reason)
                 return self.RES_LIFECYCLE_HOOK_TIMEOUT, reason
 
@@ -197,8 +195,8 @@ class ClusterAction(base.Action):
 
     def _update_nodes(self, profile_id, nodes_obj):
         # Get batching policy data if any
-        fmt = "Updating cluster '%(cluster)s': profile='%(profile)s'."
-        LOG.info(fmt, {'cluster': self.entity.id, 'profile': profile_id})
+        LOG.info("Updating cluster '%(cluster)s': profile='%(profile)s'.",
+                 {'cluster': self.entity.id, 'profile': profile_id})
         plan = []
 
         pd = self.data.get('update', None)
@@ -212,7 +210,6 @@ class ClusterAction(base.Action):
                 nodes_list.append(node.id)
             plan.append(set(nodes_list))
 
-        nodes = []
         for node_set in plan:
             child = []
             nodes = list(node_set)
@@ -308,39 +305,90 @@ class ClusterAction(base.Action):
                 ao.Action.update(self.context, action_id,
                                  {'status': base.Action.READY})
 
-    def _delete_nodes(self, node_ids):
-        action_name = consts.NODE_DELETE
+    def _remove_nodes_with_hook(self, action_name, node_ids, lifecycle_hook,
+                                inputs=None):
+        lifecycle_hook_timeout = lifecycle_hook.get('timeout')
+        lifecycle_hook_type = lifecycle_hook.get('type', None)
+        lifecycle_hook_params = lifecycle_hook.get('params')
+        if lifecycle_hook_type == "zaqar":
+            lifecycle_hook_target = lifecycle_hook_params.get('queue')
+        else:
+            # lifecycle_hook_target = lifecycle_hook_params.get('url')
+            return self.RES_ERROR, ("Lifecycle hook type '%s' is not "
+                                    "implemented") % lifecycle_hook_type
+        child = []
+        for node_id in node_ids:
+            kwargs = {
+                'name': 'node_delete_%s' % node_id[:8],
+                'cause': consts.CAUSE_DERIVED_LCH,
+                'inputs': inputs or {},
+            }
 
-        pd = self.data.get('deletion', None)
-        if pd is not None:
-            destroy = pd.get('destroy_after_deletion', True)
-            if destroy is False:
-                action_name = consts.NODE_LEAVE
+            action_id = base.Action.create(self.context, node_id, action_name,
+                                           **kwargs)
+            child.append((action_id, node_id))
 
-        # get lifecycle hook properties if specified
-        lifecycle_hook = self.data.get('hooks')
-        lifecycle_hook_timeout = None
-        if lifecycle_hook:
-            lifecycle_hook_timeout = lifecycle_hook.get('timeout')
-            lifecycle_hook_type = lifecycle_hook.get('type', None)
-            lifecycle_hook_params = lifecycle_hook.get('params')
-            if lifecycle_hook_type == "zaqar":
-                lifecycle_hook_target = lifecycle_hook_params.get('queue')
-            else:
-                # lifecycle_hook_target = lifecycle_hook_params.get('url')
-                return self.RES_ERROR, _("Lifecycle hook type '%s' is not "
-                                         "implemented") % lifecycle_hook_type
+        if child:
+            dobj.Dependency.create(self.context, [aid for aid, nid in child],
+                                   self.id)
+            # lifecycle_hook_type has to be "zaqar"
+            # post message to zaqar
+            kwargs = {
+                'user': self.context.user_id,
+                'project': self.context.project_id,
+                'domain': self.context.domain_id
+            }
 
+            notifier = msg.Message(lifecycle_hook_target, **kwargs)
+
+            for action_id, node_id in child:
+                # wait lifecycle complete if node exists and is active
+                node = no.Node.get(self.context, node_id)
+                if not node:
+                    LOG.warning('Node %s is not found. '
+                                'Skipping wait for lifecycle completion.',
+                                node_id)
+                    status = base.Action.READY
+                    child.remove((action_id, node_id))
+                elif node.status != consts.NS_ACTIVE or not node.physical_id:
+                    LOG.warning('Node %s is not in ACTIVE status. '
+                                'Skipping wait for lifecycle completion.',
+                                node_id)
+                    status = base.Action.READY
+                    child.remove((action_id, node_id))
+                else:
+                    status = base.Action.WAITING_LIFECYCLE_COMPLETION
+
+                ao.Action.update(self.context, action_id,
+                                 {'status': status})
+                if status == base.Action.WAITING_LIFECYCLE_COMPLETION:
+                    notifier.post_lifecycle_hook_message(
+                        action_id, node_id, node.physical_id,
+                        consts.LIFECYCLE_NODE_TERMINATION)
+
+            dispatcher.start_action()
+            res, reason = self._wait_for_dependents(lifecycle_hook_timeout)
+
+            if res == self.RES_LIFECYCLE_HOOK_TIMEOUT:
+                self._handle_lifecycle_timeout(child)
+
+            if res is None or res == self.RES_LIFECYCLE_HOOK_TIMEOUT:
+                dispatcher.start_action()
+                res, reason = self._wait_for_dependents()
+
+            return res, reason
+
+        return self.RES_OK, ''
+
+    def _remove_nodes_normally(self, action_name, node_ids, inputs=None):
         child = []
         for node_id in node_ids:
             kwargs = {
                 'name': 'node_delete_%s' % node_id[:8],
                 'cause': consts.CAUSE_DERIVED,
+                'inputs': inputs or {},
             }
 
-            if lifecycle_hook:
-                kwargs['cause'] = consts.CAUSE_DERIVED_LCH
-
             action_id = base.Action.create(self.context, node_id, action_name,
                                            **kwargs)
             child.append((action_id, node_id))
@@ -349,54 +397,60 @@ class ClusterAction(base.Action):
             dobj.Dependency.create(self.context, [aid for aid, nid in child],
                                    self.id)
             for action_id, node_id in child:
-                # Build dependency and make the new action ready or
-                # waiting for lifecycle completion if lifecycle hook properties
-                # are specified in deletion policy
-
-                if not lifecycle_hook:
-                    status = base.Action.READY
-                else:
-                    status = base.Action.WAITING_LIFECYCLE_COMPLETION
-
                 ao.Action.update(self.context, action_id,
-                                 {'status': status})
-                if lifecycle_hook:
-                    # lifecycle_hook_type has to be "zaqar"
-                    # post message to zaqar
-                    kwargs = {
-                        'user': self.context.user_id,
-                        'project': self.context.project_id,
-                        'domain': self.context.domain_id
-                    }
-
-                    notifier = msg.Message(lifecycle_hook_target, **kwargs)
-                    notifier.post_lifecycle_hook_message(
-                        action_id, node_id,
-                        consts.LIFECYCLE_NODE_TERMINATION)
-
-            res = None
-            if lifecycle_hook:
-                dispatcher.start_action()
-                res, reason = self._wait_for_dependents(lifecycle_hook_timeout)
-
-                if res == self.RES_LIFECYCLE_HOOK_TIMEOUT:
-                    self._handle_lifecycle_timeout(child)
-
-            if res is None or res == self.RES_LIFECYCLE_HOOK_TIMEOUT:
-                dispatcher.start_action()
-                res, reason = self._wait_for_dependents()
-
-            if res == self.RES_OK:
-                self.outputs['nodes_removed'] = node_ids
-                for node_id in node_ids:
-                    self.entity.remove_node(node_id)
-            else:
-                reason = 'Failed in deleting nodes.'
+                                 {'status': base.Action.READY})
 
+            dispatcher.start_action()
+            res, reason = self._wait_for_dependents()
             return res, reason
 
         return self.RES_OK, ''
 
+    def _delete_nodes(self, node_ids):
+        action_name = consts.NODE_DELETE
+
+        pd = self.data.get('deletion', None)
+        if pd is not None:
+            destroy = pd.get('destroy_after_deletion', True)
+            if destroy is False:
+                action_name = consts.NODE_LEAVE
+
+        stop_node_before_delete = self.entity.config.get(
+            "cluster.stop_node_before_delete", False)
+
+        # get lifecycle hook properties if specified
+        lifecycle_hook = self.data.get('hooks')
+        if lifecycle_hook:
+            if stop_node_before_delete:
+                res, reason = self._remove_nodes_with_hook(
+                    consts.NODE_OPERATION, node_ids, lifecycle_hook,
+                    {'operation': 'stop'})
+                if res != self.RES_OK:
+                    LOG.warning('Failure while stopping nodes. '
+                                'Proceed to delete nodes.')
+                res, reason = self._remove_nodes_normally(action_name,
+                                                          node_ids)
+            else:
+                res, reason = self._remove_nodes_with_hook(
+                    action_name, node_ids, lifecycle_hook)
+        else:
+            if stop_node_before_delete:
+                res, reason = self._remove_nodes_normally(
+                    consts.NODE_OPERATION, node_ids, {'operation': 'stop'})
+                if res != self.RES_OK:
+                    LOG.warning('Failure while stopping nodes. '
+                                'Proceed to delete nodes.')
+            res, reason = self._remove_nodes_normally(action_name, node_ids)
+
+        if res == self.RES_OK:
+            self.outputs['nodes_removed'] = node_ids
+            for node_id in node_ids:
+                self.entity.remove_node(node_id)
+        else:
+            reason = 'Failed in deleting nodes: %s' % reason
+
+        return res, reason
+
     @profiler.trace('ClusterAction.do_delete', hide_args=False)
     def do_delete(self):
         """Handler for the CLUSTER_DELETE action.
@@ -460,7 +514,7 @@ class ClusterAction(base.Action):
                               '' % {'n': nid, 'c': node.cluster_id})
                 continue
 
-            if node.status != "ACTIVE":
+            if node.status != consts.NS_ACTIVE:
                 errors.append('Node %s is not in ACTIVE status.' % nid)
                 continue
 
@@ -780,7 +834,8 @@ class ClusterAction(base.Action):
             dobj.Dependency.create(self.context, [c for c in children],
                                    self.id)
             for cid in children:
-                ao.Action.update(self.context, cid, {'status': 'READY'})
+                ao.Action.update(self.context, cid,
+                                 {'status': consts.ACTION_READY})
             dispatcher.start_action()
 
             # Wait for dependent action if any
@@ -919,9 +974,9 @@ class ClusterAction(base.Action):
         # desired is checked when strict is True
         curr_size = no.Node.count_by_cluster(self.context, self.target)
         if count > curr_size:
-            msg = ("Triming count (%(count)s) to current "
-                   "cluster size (%(curr)s) for scaling in")
-            LOG.warning(msg, {'count': count, 'curr': curr_size})
+            LOG.warning("Triming count (%(count)s) to current cluster size "
+                        "(%(curr)s) for scaling in",
+                        {'count': count, 'curr': curr_size})
             count = curr_size
         new_size = curr_size - count
 
@@ -1101,6 +1156,8 @@ class ClusterAction(base.Action):
             return self.RES_RETRY, 'Failed in locking cluster.'
 
         try:
+            # Refresh entity state to avoid stale data in action.
+            self.entity = cluster_mod.Cluster.load(self.context, self.target)
             res, reason = self._execute(**kwargs)
         finally:
             senlin_lock.cluster_lock_release(self.target, self.id,
