@@ -437,6 +437,12 @@ class TestString(base.SenlinTestCase):
         res = sot.to_schema_type(False)
         self.assertEqual('False', res)
 
+        res = sot.to_schema_type(None)
+        self.assertIsNone(res)
+
+        res = sot.to_schema_type(u'\u4e2d\u6587')
+        self.assertEqual(u'\u4e2d\u6587', res)
+
     def test_resolve(self):
         sot = schema.String()
 
@@ -446,6 +452,9 @@ class TestString(base.SenlinTestCase):
         res = sot.resolve(True)
         self.assertEqual('True', res)
 
+        res = sot.resolve(None)
+        self.assertIsNone(res)
+
     def test_validate(self):
         sot = schema.String()
 
